#-*- coding: utf-8 -*-

from . import Suffix

S16 = Suffix("-nDAn",     "ndan|ntan|nden|nten",      None,       True)
S7  = Suffix("-lArI",     "ları|leri",                None,       True)
S3  = Suffix("-(U)mUz",   "mız|miz|muz|müz",          "ı|i|u|ü",  True) 
S5  = Suffix("-(U)nUz",   "nız|niz|nuz|nüz",          "ı|i|u|ü",  True) 
S1  = Suffix("-lAr",      "lar|ler",                  None,       True)
S14 = Suffix("-nDA",      "nta|nte|nda|nde",          None,       True)
S15 = Suffix("-DAn",      "dan|tan|den|ten",          None,       True)
S17 = Suffix("-(y)lA",    "la|le",                    "y",        True)
S10 = Suffix("-(n)Un",    "ın|in|un|ün",              "n",        True)
S19 = Suffix("-(n)cA",    "ca|ce",                    "n",        True)
S4  = Suffix("-Un",       "ın|in|un|ün",              None,       True)
S9  = Suffix("-nU",       "nı|ni|nu|nü",              None,       True) 
S12 = Suffix("-nA",       "na|ne",                    None,       True)
S13 = Suffix("-DA",       "da|de|ta|te",              None,       True)
S18 = Suffix("-ki",       "ki",                       None,       False)
S2  = Suffix("-(U)m",     "m",                        "ı|i|u|ü",  True)
S6  = Suffix("-(s)U",     "ı|i|u|ü",                  "s",        True)
S8  = Suffix("-(y)U",     "ı|i|u|ü",                  "y",        True)
S11 = Suffix("-(y)A",     "a|e",                      "y",        True)

# The order of the enum definition determines the priority of the suffix.
# For example, -(y)ken (S15 suffix) is  checked before -n (S7 suffix).
VALUES = (S16,S7,S3,S5,S1,S14,S15,S17,S10,S19,S4,S9,S12,S13,S18,S2,S6,S8,S11)


from ..suffixes import NominalVerbSuffix
from ..transitions import Transition
from . import State

TF_VALUES, FT_VALUES, FF_VALUES = dict(), dict(), dict()

class NominalVerbState(State):
    def __init__(self, initialState, finalState, *suffixes):
        super(NominalVerbState, self).__init__(initialState, finalState, *suffixes)

        if self.initialState and (not self.finalState):
            self.VALUES = TF_VALUES
        elif (not self.initialState) and self.finalState:
            self.VALUES = FT_VALUES
        else:
            self.VALUES = FF_VALUES

    def NextState(self, suffix):
        return self.VALUES[suffix]

A = NominalVerbState(True, False, *NominalVerbSuffix.VALUES)
B = NominalVerbState(False, True, NominalVerbSuffix.S14)
C = NominalVerbState(False, True, NominalVerbSuffix.S10, NominalVerbSuffix.S12, NominalVerbSuffix.S13,NominalVerbSuffix.S14)
D = NominalVerbState(False, False, NominalVerbSuffix.S12,NominalVerbSuffix.S13)
E = NominalVerbState(False, True, NominalVerbSuffix.S1,NominalVerbSuffix.S2,NominalVerbSuffix.S3,NominalVerbSuffix.S4,NominalVerbSuffix.S5,NominalVerbSuffix.S14)
F = NominalVerbState(False, True)
G = NominalVerbState(False, False, NominalVerbSuffix.S14)
H = NominalVerbState(False, False, NominalVerbSuffix.S1,NominalVerbSuffix.S2,NominalVerbSuffix.S3,NominalVerbSuffix.S4,NominalVerbSuffix.S5,NominalVerbSuffix.S14)

ALL = (A,B,C,D,E,F,G,H)

for sfx in (NominalVerbSuffix.S1,NominalVerbSuffix.S2,NominalVerbSuffix.S3,NominalVerbSuffix.S4):
    TF_VALUES[sfx] = B

TF_VALUES[NominalVerbSuffix.S5] = C

for sfx in (NominalVerbSuffix.S6,NominalVerbSuffix.S7,NominalVerbSuffix.S8,NominalVerbSuffix.S9):
    TF_VALUES[sfx] = D

TF_VALUES[NominalVerbSuffix.S10] = E

for sfx in (NominalVerbSuffix.S12,NominalVerbSuffix.S13,NominalVerbSuffix.S14,NominalVerbSuffix.S15):
    TF_VALUES[sfx] = F

TF_VALUES[NominalVerbSuffix.S11] = H

for sfx in (NominalVerbSuffix.S1,NominalVerbSuffix.S2,NominalVerbSuffix.S3,NominalVerbSuffix.S4,NominalVerbSuffix.S5):
    FT_VALUES[sfx] = G

for sfx in (NominalVerbSuffix.S10,NominalVerbSuffix.S12,NominalVerbSuffix.S13,NominalVerbSuffix.S14):
    FT_VALUES[sfx] = F

for sfx in (NominalVerbSuffix.S1,NominalVerbSuffix.S2,NominalVerbSuffix.S3,NominalVerbSuffix.S4,NominalVerbSuffix.S5):
    FF_VALUES[sfx] = G

for sfx in (NominalVerbSuffix.S12,NominalVerbSuffix.S13,NominalVerbSuffix.S14):
    FF_VALUES[sfx] = F


from ..transitions import Transition
from ..suffixes import *

__all__ = ["State"]

class State(object):
    def __init__(self, initialState, finalState, *suffixes):
        self.initialState = initialState
        self.finalState = finalState
        if suffixes is None:
            self.suffixes = ()
        else:
            self.suffixes = suffixes

    def AddTransitions(self, word, transitions, marked):
        for suffix in self.suffixes:
            if suffix.Match(word):
                transitions.append(Transition(self, self.NextState(suffix), word, suffix, marked))

    def NextState(self, suffix):
        raise NotImplementedError("Feature is not implemented.")


import re

__all__ = ["Suffix"]

class Suffix(object):
    def __init__(self, name, pattern, optionalLetter, checkHarmony):
        self.name = name
        self.pattern = re.compile("(" + pattern + ")$", re.UNICODE)

        if optionalLetter is None:
            self.optionalLetterCheck = False
            self._optionalLetterPattern = None
        else:
            self.optionalLetterCheck = True
            self._optionalLetterPattern = re.compile("(" + optionalLetter + ")$", re.UNICODE)

        self.checkHarmony = checkHarmony

    def Match(self, word):
        return self.pattern.search(word)

    def OptionalLetter(self, word):
        if self.optionalLetterCheck:
            match = self._optionalLetterPattern.search(word)
            if match:
                return match.group()

    def RemoveSuffix(self, word):
        return self.pattern.sub("", word)

    @property
    def CheckHarmony(self):
        return self.checkHarmony




__all__ = ["Transition"]

class Transition(object):
    def __init__(self, startState, nextState, word, suffix, marked):
        self.startState = startState
        self.nextState = nextState
        self.word = word
        self.suffix = suffix
        self.marked = False

    def similarTransitions(self, transitions):
        for transition in transitions:
            if (self.startState == transition.startState and 
                self.nextState == transition.nextState):
                yield transition


from ..suffixes import NounSuffix
from ..transitions import Transition
from . import State

TT_VALUES, FT_VALUES, FF_VALUES = dict(), dict(), dict()

class NounState(State):
    def __init__(self, initialState, finalState, *suffixes):
        super(NounState, self).__init__(initialState, finalState, *suffixes)

        if self.initialState and self.finalState:
            self.VALUES = TT_VALUES
        elif (not self.initialState) and self.finalState:
            self.VALUES = FT_VALUES
        else:
            self.VALUES = FF_VALUES

    def NextState(self, suffix):
        return self.VALUES[suffix]

A = NounState(True, True, *NounSuffix.VALUES)
B = NounState(False, True, NounSuffix.S1, NounSuffix.S2, NounSuffix.S3, NounSuffix.S4, NounSuffix.S5)
C = NounState(False, False, NounSuffix.S6, NounSuffix.S7)
D = NounState(False, False, NounSuffix.S10, NounSuffix.S13, NounSuffix.S14)
E = NounState(False, True, NounSuffix.S1, NounSuffix.S2, NounSuffix.S3, NounSuffix.S4, NounSuffix.S5, NounSuffix.S6, NounSuffix.S7, NounSuffix.S18)
F = NounState(False, False, NounSuffix.S6, NounSuffix.S7, NounSuffix.S18)
G = NounState(False, True, NounSuffix.S1, NounSuffix.S2, NounSuffix.S3, NounSuffix.S4, NounSuffix.S5, NounSuffix.S18)
H = NounState(False, True, NounSuffix.S1)
K = NounState(False, True)
L = NounState(False, True, NounSuffix.S18)
M = NounState(False, True, NounSuffix.S1, NounSuffix.S2, NounSuffix.S3, NounSuffix.S4, NounSuffix.S5, NounSuffix.S6, NounSuffix.S6, NounSuffix.S7)

ALL = (A,B,C,D,E,F,G,H,K,L,M)

for sfx in (NounSuffix.S8,NounSuffix.S11,NounSuffix.S13):
    TT_VALUES[sfx] = B

for sfx in (NounSuffix.S9, NounSuffix.S16):
    TT_VALUES[sfx] = C

FF_VALUES[NounSuffix.S18] = FT_VALUES[NounSuffix.S18] = TT_VALUES[NounSuffix.S18] = D

for sfx in (NounSuffix.S10, NounSuffix.S17):
    TT_VALUES[sfx] = E

for sfx in (NounSuffix.S12,NounSuffix.S14):
    TT_VALUES[sfx] = F

TT_VALUES[NounSuffix.S15] = G

for sfx in (NounSuffix.S2,NounSuffix.S3,NounSuffix.S4,NounSuffix.S5,NounSuffix.S6):
    FT_VALUES[sfx] = TT_VALUES[sfx] = H

FF_VALUES[NounSuffix.S7] = FT_VALUES[NounSuffix.S7] = TT_VALUES[NounSuffix.S7] = K
FT_VALUES[NounSuffix.S1] = TT_VALUES[NounSuffix.S1] = L
TT_VALUES[NounSuffix.S19] = M

FF_VALUES[NounSuffix.S13] = B
FF_VALUES[NounSuffix.S10] = E
FF_VALUES[NounSuffix.S14] = F
FF_VALUES[NounSuffix.S6] = H


#-*- coding: utf-8 -*-

from . import Suffix

S11 = Suffix("-cAsInA",    "casına|çasına|cesine|çesine",      None, True)
S4  = Suffix("-sUnUz",     "sınız|siniz|sunuz|sünüz",          None, True)
S14 = Suffix("-(y)mUş",    "muş|miş|müş|mış",                  "y",  True)
S15 = Suffix("-(y)ken",    "ken",                              "y",  True)
S2  = Suffix("-sUn",       "sın|sin|sun|sün",                  None, True)
S5  = Suffix("-lAr",       "lar|ler",                          None, True)
S9  = Suffix("-nUz",       "nız|niz|nuz|nüz",                  None, True)
S10 = Suffix("-DUr",       "tır|tir|tur|tür|dır|dir|dur|dür",  None, True)
S3  = Suffix("-(y)Uz",     "ız|iz|uz|üz",                      "y",  True)
S1  = Suffix("-(y)Um",     "ım|im|um|üm",                      "y",  True)
S12 = Suffix("-(y)DU",     "dı|di|du|dü|tı|ti|tu|tü",          "y",  True)
S13 = Suffix("-(y)sA",     "sa|se",                            "y",  True)
S6  = Suffix("-m",         "m",                                None, True)
S7  = Suffix("-n",         "n",                                None, True)
S8  = Suffix("-k",         "k",                                None, True)

# The order of the enum definition determines the priority of the suffix.
# For example, -(y)ken (S15 suffix) is  checked before -n (S7 suffix).
VALUES = (S11,S4,S14,S15,S2,S5,S9,S10,S3,S1,S12,S13,S6,S7,S8)


#!/usr/bin/env python

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='TurkishStemmer',
    version='1.1',
    description='Turkish Stemmer',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Hanefi Önaldı',
    author_email='abdullahanefi16@gmail.com',
    url='https://github.com/hanefi/turkish-stemmer-python',
    packages=find_packages(),
    )



from ..suffixes import DerivationalSuffix
from ..transitions import Transition
from . import State

class DerivationalState(State):
    def __init__(self, initialState, finalState, *suffixes):
        super(DerivationalState, self).__init__(initialState, finalState, *suffixes)

    def NextState(self, suffix):
        if self.initialState:
            return B

A = DerivationalState(True, False, *DerivationalSuffix.VALUES)
B = DerivationalState(False, True)